/*
 * Copyright (C) 2012  Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "calendaringtest.h"

#include <QTest>
#include <kolabevent.h>
#include <iostream>
#include <calendaring/event.h>
#include <calendaring/datetimeutils.h>

#include "testhelpers.h"
#include "testutils.h"

void compareEvents(const std::vector<Kolab::Event> &list1, const std::vector<Kolab::Event> &list2)
{
    QCOMPARE(list1.size(), list2.size());
    for (std::size_t i = 0; i < list1.size(); i++) {
        const Kolab::Event &e1 = list1.at(i);
        const Kolab::Event &e2 = list2.at(i);
//         qDebug() << i;
//         QCOMPARE(e1.uid(), e2.uid());
        QCOMPARE(e1.start(), e2.start());
        QCOMPARE(e1.end(), e2.end());
    }
}

void CalendaringTest::initTestCase()
{
}

void CalendaringTest::testCalendaringEvent()
{
    Kolab::Event event;
    event.setUid("uid");
    event.setStart(Kolab::cDateTime(2011,10,10,12,1,1,true));
    event.setEnd(Kolab::cDateTime(2011,10,11,12,1,1,true));
    Kolab::Calendaring::Event calEvent(event);

    QCOMPARE(event.start(), calEvent.start());
    QCOMPARE(event.uid(), calEvent.uid());

    Kolab::Calendaring::Event calEvent2;
    Kolab::Calendaring::Event calEvent3 = calEvent2;
    QVERIFY(!calEvent2.uid().empty());
    QCOMPARE(calEvent2.uid(), calEvent3.uid());
}

void CalendaringTest::delegationTest()
{
    Kolab::Calendaring::Event event;
    event.setStart(Kolab::cDateTime("Europe/Zurich",2012,5,5,3,4,4));

    Kolab::Attendee att1(Kolab::ContactReference("email1", "name1", "uid1"));
    att1.setCutype(Kolab::CutypeIndividual);
    Kolab::Attendee att2(Kolab::ContactReference("email2", "name2", "uid2"));
    Kolab::Attendee att3(Kolab::ContactReference("email3", "name3", "uid3"));
    Kolab::Attendee att4(Kolab::ContactReference("email4", "name4", "uid4"));
    
    std::vector <Kolab::Attendee > attendees;
    attendees.push_back(att1);
    attendees.push_back(att2);
    attendees.push_back(att3);
    event.setAttendees(attendees);

    std::vector <Kolab::Attendee > delegators;
    delegators.push_back(att1);
    delegators.push_back(att2);

    std::vector <Kolab::Attendee > delegatees;
    delegatees.push_back(att3);
    delegatees.push_back(att4);

    event.delegate(delegators, delegatees);

    std::cout << event.write();
    //TODO write an actual test
}

void CalendaringTest::testRecurrence()
{
    //FIXME this is currently broken for floating or utc,
    //because KCalendarCore::RecurrenceRule::getNextDate implicitly
    //converts to the timezone of dtstart, which defaults to the local
    //timezone if none is set.
    Kolab::Calendaring::Event event;
    event.setStart(Kolab::cDateTime("Europe/Zurich", 2011,1,1,1,1,1));
    event.setEnd(Kolab::cDateTime("Europe/Zurich", 2011,1,1,2,1,1));
    Kolab::RecurrenceRule rrule;
    rrule.setFrequency(Kolab::RecurrenceRule::Daily);
    rrule.setInterval(1);
    rrule.setCount(10);
    event.setRecurrenceRule(rrule);

    Kolab::cDateTime previousDate = event.start();
    for (int i = 0; i < 9; i++) {
        const Kolab::cDateTime nextDate = event.getNextOccurence(previousDate);
        const char *nextDateString = QTest::toString(nextDate);
        qDebug() << nextDateString;
        delete[] nextDateString;
        QCOMPARE(nextDate, Kolab::cDateTime("Europe/Zurich", previousDate.year(), previousDate.month(), previousDate.day()+1, previousDate.hour(), previousDate.minute(), previousDate.second()));
        const Kolab::cDateTime endDate = event.getOccurenceEndDate(nextDate);
//         qDebug() << QTest::toString(endDate);
        QCOMPARE(endDate, Kolab::cDateTime("Europe/Zurich", nextDate.year(), nextDate.month(), nextDate.day(), event.end().hour(), event.end().minute(), event.end().second()));
        previousDate = nextDate;
    }
    
    Kolab::cDateTime outOfScopeDate = event.getNextOccurence(previousDate);
    QVERIFY(!outOfScopeDate.isValid());
        
}

void CalendaringTest::testDateTimeUtils()
{
    std::cout << Kolab::DateTimeUtils::getLocalTimezone() << std::endl;
}

void CalendaringTest::testICal()
{
    Kolab::Calendaring::Event event;
    event.setStart(Kolab::cDateTime("Europe/Zurich",2012,5,5,3,4,4));
    const std::string &result = event.toICal();
    //TODO write an actual test
    event.setStart(Kolab::cDateTime(1,1,1));
    event.fromICal(result);
    QCOMPARE(event.start(), Kolab::cDateTime("Europe/Zurich",2012,5,5,3,4,4));
}


void CalendaringTest::testParseICal()
{
    QFile icalFile( getPath("v3/event/simple.ics") );
    QVERIFY( icalFile.open( QFile::ReadOnly ) );
    Kolab::Calendaring::Event event;
    event.fromICal(icalFile.readAll().constData());
    QCOMPARE(event.start(), Kolab::cDateTime("Europe/Berlin", 2009, 9, 2, 8, 0, 0));
}


QTEST_MAIN( CalendaringTest )

#include "calendaringtest.moc"
